<?php
declare(strict_types = 1);

require_once(__DIR__ . '/Admin.class.php');

/**
 * Gestion des albums.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class AdminAlbums extends Admin
{
	/**
	 * Informations en base de données de la catégorie courante.
	 *
	 * @var array
	 */
	public static $catInfos = [];



	/**
	 * Rognage d'une vignette.
	 *
	 * @return void
	 */
	public static function cropThumb(): void
	{
		if (!isset($_POST['thumb_edit']) || !isset($_POST['crop_selection'])
		|| !preg_match('`^\{"x":\d{1,5},"y":\d{1,5},"w":\d{1,5},"h":\d{1,5}\}$`',
		$_POST['crop_selection']))
		{
			return;
		}

		// Identifiant de l'objet.
		$id = $_GET['item_id'] ?? $_GET['category_id'];

		// Récupération des informations utiles de l'image.
		if (isset($_GET['item_id']))
		{
			$sql = "SELECT item_tb_params AS tb_params,
						   item_width,
						   item_height
					  FROM {items}
					 WHERE item_id = ?";
		}
		else
		{
			$sql = "SELECT cat_tb_params AS tb_params,
						   item_width,
						   item_height
					  FROM {categories} AS cat
				 LEFT JOIN {items} AS i
					    ON cat.thumb_id = i.item_id
					 WHERE cat.cat_id = ?";
		}
		if (!DB::execute($sql, $id))
		{
			Report::error();
		}
		if (!$i = DB::fetchRow())
		{
			return;
		}

		// Si la sélection couvre toute l'image,
		// on supprime les paramètres de sélection.
		$selection = json_decode($_POST['crop_selection'], TRUE);
		if ($selection['x'] == 0 && $selection['y'] == 0
		&& $selection['h'] == $i['item_height'] && $selection['w'] == $i['item_width'])
		{
			$_POST['crop_selection'] = NULL;
		}

		// Si les paramètres de sélection sont identiques,
		// inutile d'aller plus loin.
		if ($i['tb_params'] === $_POST['crop_selection'])
		{
			Report::noChange();
			return;
		}

		// Mise à jour de la base de données.
		if (isset($_GET['item_id']))
		{
			$sql = "UPDATE {items}
					   SET item_tb_params = :crop
					 WHERE item_id = :id";
		}
		else
		{
			$sql = "UPDATE {categories}
					   SET cat_tb_params = :crop
					 WHERE cat_id = :id";
		}
		if (!DB::execute($sql, ['crop' => $_POST['crop_selection'], 'id' => $id]))
		{
			Report::error();
			return;
		}

		Report::success();
	}

	/**
	 * Construit la liste de toutes les catégories de la galerie.
	 *
	 * @return void
	 */
	public static function getCategoriesLists(): void
	{
		$category_one_sections = ['album-sort', 'category-sort', 'category-items'];
		$get_url = function($page) use ($category_one_sections)
		{
			switch ($_GET['section'])
			{
				case 'album-delete' :
				case 'album-edit' :
				case 'album-geolocation' :
				case 'album-sort' :
				case 'album-thumb' :
				case 'album-thumb-new' :
				case 'category-delete' :
				case 'category-edit' :
				case 'category-geolocation' :
				case 'category-items' :
				case 'category-sort' :
				case 'category-thumb' :
				case 'category-thumb-new' :
					if ($page == 'category/1' &&
					!in_array($_GET['section'], $category_one_sections))
					{
						break;
					}
					$section = preg_replace('`^(album|category)\-`', '', $_GET['section']);
					$page = str_replace('album/', 'album-' . $section . '/', $page);
					$page = str_replace('category/', 'category-' . $section . '/', $page);
					$page = str_replace('album-items', 'album', $page);
					break;
			}
			$filter = isset($_GET['filter'])
				? '/' . $_GET['filter'] . '/' . $_GET['filter_value']
				: '';

			return App::getURL($page . $filter);
		};
		$categories = Category::getList($makelist, $get_url, $_GET['category_id'] ?? 1);
		if (!$makelist)
		{
			return;
		}
		$list = $makelist($categories);
		Template::set('categories_move', $list);

		// Réduction de la liste "parcourir".
		$reduce_from_items = function() use (&$categories, &$makelist)
		{
			if (!(($_GET['filter'] == 'search' && $where = self::$search->sql())
			|| $where = SQL::itemsWhere($_GET['filter'], $_GET['filter_value'])))
			{
				return;
			}
			$sql_from = SQL::itemsFrom($_GET['filter'] ?? '');
			$sql = "SELECT cat.cat_id
					  FROM {items} AS i
						   $sql_from
				 LEFT JOIN {categories} AS cat
					    ON cat.cat_id = i.cat_id
					 WHERE " . $where['sql'];
			if (!DB::execute($sql, $where['params']))
			{
				return;
			}
			$parents_id = DB::fetchCol('cat_id');

			Category::reduceList('album', $categories, $parents_id);
			Template::set('categories_browse', $makelist($categories));
		};
		$reduce_from_categories = function() use (&$categories, &$makelist)
		{
			if (!$where = self::sqlWhereCategories())
			{
				return;
			}
			$sql = "SELECT parent_id AS cat_id
					  FROM {categories} AS cat
					 WHERE " . $where['sql'];
			if (!DB::execute($sql, $where['params']))
			{
				return;
			}
			$parents_id = DB::fetchCol('cat_id');

			Category::reduceList('category', $categories, $parents_id);
			Template::set('categories_browse', $makelist($categories));
		};
		if (isset($_GET['filter']))
		{
			if (isset($_GET['search']))
			{
				if (self::$search->options['type'] == 'items')
				{
					$reduce_from_items();
				}
				else
				{
					$reduce_from_categories();
				}
			}
			else
			{
				$reduce_from_items();
			}
		}
		else
		{
			Template::set('categories_browse', $list);
		}
	}

	/**
	 * Récupère les paramètres de rognage d'une image.
	 *
	 * @params string $type
	 *   Type : 'cat' ou 'item'.
	 *
	 * @return void
	 */
	public static function getCropInfos(string $type): void
	{
		if ($type == 'cat')
		{
			$i =& AdminAlbums::$catInfos;
		}
		else
		{
			$i =& AdminItems::$infos;
		}

		$selection = json_decode($i['cat_tb_params'] ?? $i['item_tb_params'] ?? '', TRUE);

		if (Item::isImage($i['item_type']))
		{
			$image = $i['item_path'];
			$width = $i['item_width'];
			$height = $i['item_height'];
		}
		else
		{
			$filename = Video::getCapture((int) $i['item_id'], $i['item_adddt']);
			$image = '/cache/captures/' . $filename;
			if (file_exists(GALLERY_ROOT . $image))
			{
				$image_size = Image::getTypeSize(GALLERY_ROOT . $image);
			}
			$width = $image_size['width'] ?? 0;
			$height = $image_size['height'] ?? 0;
		}

		Template::set('crop',
		[
			'image' => App::getFileSource($image),
			'width' => $width,
			'height' => $height,
			'selection_x' => $selection['x'] ?? 0,
			'selection_y' => $selection['y'] ?? 0,
			'selection_w' => $selection['w'] ?? $width,
			'selection_h' => $selection['h'] ?? $height
		]);
	}

	/**
	 * Récupère tous les tags de la galerie.
	 *
	 * @return array
	 */
	public static function getTagsList(): array
	{
		Template::set('tags', []);
		$sql = "SELECT tag_id, tag_name FROM {tags} ORDER BY LOWER(tag_name) ASC";
		if (!DB::execute($sql))
		{
			return [];
		}
		if (!$tags = DB::fetchAll('tag_id', 'tag_name'))
		{
			return [];
		}

		$tags = array_values($tags);
		Template::set('tags', $tags);

		return $tags;
	}

	/**
	 * Retourne l'URL de la catégorie 1 de la section courante.
	 *
	 * @return void
	 */
	public static function getURLSectionCatOne(): string
	{
		$section = $_GET['section'] == 'album' ? 'category' : $_GET['section'];
		$url = preg_replace('`^[^/]+/\d+`', $section . '/1', $_GET['q_pageless']);

		return $url;
	}

	/**
	 * Retourne la clause WHERE correspondant
	 * aux critères de recherche des catégories à récupérer.
	 *
	 * @return array
	 */
	public static function sqlWhereCategories(): array
	{
		$search = self::$search->sql();
		if (!$search)
		{
			App::redirect(preg_replace('`/search/[^/]+`', '', $_GET['q_pageless']));
			return [];
		}

		return $search;
	}

	/**
	 * Informations de template pour les filtres.
	 *
	 * @return void
	 */
	public static function tplFilter(): void
	{
		Template::set('filter', [
			'exit_link' => App::getURL($_GET['q_filterless']),
			'exit_text' => __('Montrer tous les fichiers')
		]);
		switch ($_GET['filter'])
		{
			// Appareils photos.
			case 'camera-brand' :
				$sql = "SELECT camera_brand_name FROM {cameras_brands} WHERE camera_brand_id = ?";
				DB::execute($sql, (int) $_GET['filter_value']);
				if (!$brand_name = DB::fetchVal())
				{
					App::redirect($_GET['q_filterless']);
				}
				Template::set('filter', [
					'text' => __('Photos prises par un appareil photos de marque %s'),
					'text_format_text' => $brand_name
				]);
				break;

			case 'camera-model' :
				$sql = "SELECT camera_model_name FROM {cameras_models} WHERE camera_model_id = ?";
				DB::execute($sql, (int) $_GET['filter_value']);
				if (!$model_name = DB::fetchVal())
				{
					App::redirect($_GET['q_filterless']);
				}
				Template::set('filter', [
					'text' => __('Photos prises avec le modèle d\'appareil photos %s'),
					'text_format_text' => $model_name
				]);
				break;

			// Tags.
			case 'tag' :
				$sql = "SELECT tag_name FROM {tags} WHERE tag_id = ?";
				DB::execute($sql, (int) $_GET['filter_value']);
				if (!$tag_name = DB::fetchVal())
				{
					App::redirect($_GET['q_filterless']);
				}
				Template::set('filter', [
					'text' => __('Fichiers liés au tag %s'),
					'text_format_text' => $tag_name
				]);
				break;

			// Fichiers de l'utilisateur.
			case 'user-favorites' :
				$text = $text ?? __('Favoris de %s');
			case 'user-images' :
				$text = $text ?? __('Photos de %s');
			case 'user-items' :
				$text = $text ?? __('Fichiers de %s');
			case 'user-videos' :
				$text = $text ?? __('Vidéos de %s');
				$sql = "SELECT user_login FROM {users} WHERE user_id = ?";
				DB::execute($sql, (int) $_GET['filter_value']);
				if (!$user_login = DB::fetchVal())
				{
					App::redirect($_GET['q_filterless']);
				}
				Template::set('filter', [
					'text' => $text,
					'text_format_link' => App::getURL(
						'user/' . (int) $_GET['filter_value']
					),
					'text_format_text' => $user_login
				]);
				break;
		}
	}

	/**
	 * Informations de template pour le moteur de recherche.
	 *
	 * @param array $functions
	 *
	 * @return void
	 */
	public static function tplSearch(array &$functions): void
	{
		// Type d'objet.
		$type_categories = isset(self::$search->options['type'])
			&& self::$search->options['type'] == 'categories';
		$type_items = isset(self::$search->options['type'])
			&& self::$search->options['type'] == 'items';

		// Colonnes.
		$columns_categories =
		[
			'cat_desc'  => TRUE,
			'cat_name'  => TRUE,
			'cat_path'  => FALSE,
			'cat_url'   => TRUE
		];
		$columns_items =
		[
			'item_desc' => TRUE,
			'item_name' => TRUE,
			'item_path' => FALSE,
			'item_url'  => TRUE
		];
		if ($type_items)
		{
			$columns = array_merge(
				$functions['columns']($columns_items),
				$columns_categories
			);
		}
		else
		{
			$columns = array_merge(
				$functions['columns']($columns_categories),
				$columns_items
			);
		}

		// Exclusion.
		$item_exclude_column = $functions['select']('item_exclude_column',
		[
			[
				'value' => 'comments',
				'text' => __('Commentaires')
			],
			[
				'value' => 'crtdt',
				'text' => __('Date de création')
			],
			[
				'value' => 'desc',
				'text' => __('Description')
			],
			[
				'value' => 'geoloc',
				'text' => __('Géolocalisation')
			],
			[
				'value' => 'hits',
				'text' => __('Vues')
			],
			[
				'value' => 'tags',
				'text' => __('Tags')
			],
			[
				'value' => 'votes',
				'text' => __('Votes')
			]
		]);

		// Type de fichier.
		$item_type = $functions['select']('item_type',
		[
			[
				'value' => 'all',
				'text' => __('*Tous')
			],
			[
				'value' => 'image',
				'text' => __('Image (tous les formats)')
			],
			[
				'value' => 'video',
				'text' => __('Vidéo (tous les formats)')
			],
			[
				'value' => 'image_gif',
				'text' => Item::getTypeText(Item::TYPE_GIF)
			],
			[
				'value' => 'image_gif_animated',
				'text' => Item::getTypeText(Item::TYPE_GIF_ANIMATED)
			],
			[
				'value' => 'image_jpeg',
				'text' => Item::getTypeText(Item::TYPE_JPEG)
			],
			[
				'value' => 'image_png',
				'text' => Item::getTypeText(Item::TYPE_PNG)
			],
			[
				'value' => 'image_png_animated',
				'text' => Item::getTypeText(Item::TYPE_PNG_ANIMATED)
			],
			[
				'value' => 'image_webp',
				'text' => Item::getTypeText(Item::TYPE_WEBP)
			],
			[
				'value' => 'image_webp_animated',
				'text' => Item::getTypeText(Item::TYPE_WEBP_ANIMATED)
			],
			[
				'value' => 'video_mp4',
				'text' => Item::getTypeText(Item::TYPE_MP4)
			],
			[
				'value' => 'video_webm',
				'text' => Item::getTypeText(Item::TYPE_WEBM)
			]
		]);

		// État.
		$status = $functions['select']('status',
		[
			[
				'value' => '*',
				'text' => __('*Tous')
			],
			[
				'value' => '1',
				'text' => __('Activé')
			],
			[
				'value' => '0',
				'text' => __('Désactivé')
			]
		]);

		Template::set('search', $functions['date']());
		Template::set('search',
		[
			'columns' => $columns,
			'date' => self::$search->options['date'] ?? FALSE,
			'date_column_cat' => $type_categories && isset(self::$search->options['date_column'])
				? self::$search->options['date_column']
				: 'cat_crtdt',
			'date_column_items' => $type_items && isset(self::$search->options['date_column'])
				? self::$search->options['date_column']
				: 'item_crtdt',
			'item_exclude' => self::$search->options['item_exclude'] ?? FALSE,
			'item_exclude_column' => $item_exclude_column,
			'item_filesize' => self::$search->options['item_filesize'] ?? FALSE,
			'item_filesize_end' => self::$search->options['item_filesize_end'] ?? '',
			'item_filesize_start' => self::$search->options['item_filesize_start'] ?? '',
			'item_size' => self::$search->options['item_size'] ?? FALSE,
			'item_size_height_end' => self::$search->options['item_size_height_end'] ?? '',
			'item_size_height_start' => self::$search->options['item_size_height_start'] ?? '',
			'item_size_width_end' => self::$search->options['item_size_width_end'] ?? '',
			'item_size_width_start' => self::$search->options['item_size_width_start'] ?? '',
			'item_tags' => self::$search->options['item_tags'] ?? TRUE,
			'item_type' => $item_type,
			'status' => $status,
			'type' =>
			[
				'categories' => $type_categories,
				'items' => self::$search->query === '' || $type_items
			],
			'user' => self::$search->options['user'] ?? ''
		]);
	}

	/**
	 * Nouvelle capture d'image d'une vidéo.
	 *
	 * @return void
	 */
	public static function videoCapture(): void
	{
		if (!isset($_POST['new_thumb']) || !isset($_POST['video_capture'])
		 || !isset($_POST['video_hash']) || !isset($_POST['video_id'])
		 || !isset($_POST['video_width']) || !isset($_POST['video_height'])
		 || !isset($_POST['video_duration']))
		{
			return;
		}

		$data = base64_decode($_POST['video_capture']);
		$capture = GALLERY_ROOT . '/cache/captures/'
			. $_POST['video_id'] . '-' . $_POST['video_hash'] . '.jpg';

		if (File::putContents($capture, $data) !== FALSE)
		{
			// Suppression des images redimensionnées.
			Item::deleteResized([$_POST['video_id']], ['resize', 'thumbs']);

			// Enregistrement des paramètres de la vidéo en base de données.
			$duration = (int) $_POST['video_duration'];
			$width = (int) $_POST['video_width'];
			$height = (int) $_POST['video_height'];
			if ($width > 0 && $height > 0 && $duration > 0)
			{
				$sql = "UPDATE {items}
						   SET item_duration = :duration,
							   item_width = :width,
							   item_height = :height
						 WHERE item_id = :id";
				$params =
				[
					'duration' => $duration,
					'height' => $height,
					'id' => $_POST['video_id'],
					'width' => $width
				];
				if (!DB::execute($sql, $params))
				{
					Report::error();
					return;
				}
			}

			Report::success();
		}
		else
		{
			Report::error();
		}
	}



	/**
	 * Retourne le paramètre $param de la catégorie parente
	 * sur laquelle a été défini un mot de passe.
	 *
	 * @param string $param
	 *   Nom du paramètre à retourner.
	 * @param array $object_infos
	 *   Informations de l'objet.
	 * @param array $parents_infos
	 *   Informations des parents de l'objet.
	 * @param array $is_item
	 *   L'objet est-il un fichier ?
	 *
	 * @return mixed
	 */
	protected static function _getPasswordParent(string $param,
	array &$object_infos, array &$parents_infos, bool $is_item)
	{
		if ($object_infos['password_id'] === NULL)
		{
			return NULL;
		}

		$parent_password = $is_item || $object_infos['p_cat_id'] != $object_infos['cat_id'];

		$i = $parent_password && array_key_exists($object_infos['p_cat_id'], $parents_infos)
			? $parents_infos[$object_infos['p_cat_id']]
			: NULL;

		switch ($param)
		{
			case 'link' :
				return $i ? App::getURL($i['cat_type'] . '/' . $i['cat_id']) : '';

			case 'name' :
			case 'type' :
				return $i ? $i['cat_' . $param] : '';

			default :
				return $parent_password;
		}
	}
}
?>