<?php
declare(strict_types = 1);

/**
 * Méthodes de base de données pour SQLite.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class DBLayer
{
	/**
	 * Nom du SGBD.
	 *
	 * @var string
	 */
	const NAME = 'SQLite';

	/**
	 * Numéro de port du serveur par défaut.
	 *
	 * @var string
	 */
	const PORT_DEFAULT = '';

	/**
	 * Version minimale requise.
	 *
	 * @var string
	 */
	const VERSION_MIN = '3';



	/**
	 * Retourne des informations sur la base de données.
	 *
	 * @return array
	 */
	public static function getDetails(): array
	{
		$details =
		[
			'filesize' =>
			[
				'text' => __('Taille de la base de données'),
				'value' => L10N::formatFilesize(filesize(CONF_DB_NAME))
			]
		];

		return $details;
	}



	/**
	 * Retourne le DSN pour se connecter à la base de données.
	 *
	 * @return string
	 */
	protected static function _getDSN(): string
	{
		if (CONF_ACCESS_KEY && !file_exists(CONF_DB_NAME))
		{
			die('No database.');
		}
		return CONF_DB_TYPE . ':' . CONF_DB_NAME;
	}

	/**
	 * Paramètres d'initialisation.
	 *
	 * @param PDO $pdo
	 *
	 * @return bool
	 */
	protected static function _initialize(PDO $pdo): bool
	{
		// Création de la fonction REGEXP.
		$sqlite_regexp = function(string $pattern, $value): bool
		{
			if ($value === NULL)
			{
				return FALSE;
			}
			$pattern = str_replace(
				['[:alnum:]', '[:space:]', '([[:<:]]|^)', '([[:>:]]|$)'],
				['[a-z0-9]', '\s', '\b', '\b'],
				$pattern
			);

			#mb_regex_encoding('UTF-8');
			return (bool) mb_eregi($pattern, (string) $value);
		};
		if (!$pdo->sqliteCreateFunction('regexp', $sqlite_regexp, 2))
		{
			return FALSE;
		}

		// On booste les performances.
		// https://sqlite.org/pragma.html#pragma_synchronous
		if (!DB::execute("PRAGMA synchronous = OFF"))
		{
			return FALSE;
		}

		// Activation des clés étrangères.
		if (!DB::execute("PRAGMA foreign_keys = ON"))
		{
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Nom de la séquence d'objets (uniquement pour PostgreSQL).
	 *
	 * @param array $seq
	 *
	 * @return null
	 */
	protected static function _seqName(array $seq = [])
	{
		return NULL;
	}

	/**
	 * Code SQL à remplacer.
	 *
	 * @param string $sql
	 *
	 * @return string
	 */
	protected static function _replaceSQL(string $sql): string
	{
		// Remplacement de la fonction NOW().
		if (CONF_DB_DATE == 'php')
		{
			$sql = str_replace("NOW()", "'" . date('Y-m-d H:i:s') . "'", $sql);
		}

		// Addition de temps.
		$sql = preg_replace('`DATE_ADD\(([^,]+),\s*INTERVAL\s+([^\)]+)\)`',
			'datetime($1, \'$2\')', $sql);

		// Opérateur LIKE.
		if (strstr($sql, 'LIKE'))
		{
			$likes = '(?:\?|:?[0-9a-z_]+|\'.+?(?<!\x5c)\')';
			$sql = preg_replace(
				"`(LIKE\s+$likes(?:\s*\|\|\s*$likes)*)`i",
				"$1 ESCAPE '\x5c'",
				$sql
			);
		}

		// Ne pas modifier l'ordre des textes à rechercher !
		$sql = str_replace(
			[
				"RAND()",
				"INSERT IGNORE",
				"UPDATE IGNORE",
				"AUTO_INCREMENT",
				"NOW()",
				"TO_SECONDS(",
				"datetime(datetime('now')"
			],
			[
				"RANDOM()",
				"INSERT OR IGNORE",
				"UPDATE OR IGNORE",
				"AUTOINCREMENT",
				"datetime('now')",
				"strftime('%s', ",
				"datetime('now'"
			],
			$sql
		);

		return $sql;
	}
}
?>