<?php
declare(strict_types = 1);

require_once(__DIR__ . '/Gallery.class.php');

/**
 * Opérations concernant les fichiers.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class GalleryItems extends Gallery
{
	/**
	 * Informations en base de données du fichier courant.
	 *
	 * @var array
	 */
	public static $infos = [];



	/**
	 * Récupération des fichiers de l'album courant.
	 *
	 * @return int
	 */
	public static function getAlbumItems(): int
	{
		self::_pagination(1);

		// Nombre de fichiers par page.
		$items_per_page = (int) Config::$params['thumbs_item_nb_per_page'];
		Template::set('items_per_page', $items_per_page);

		// Récupération des informations des fichiers.
		if (is_int($items = self::_getItems($items_per_page)))
		{
			return $items;
		}

		self::_pagination(ceil(Template::$data['objects_count'] / $items_per_page));

		// Formatage des informations.
		$formated_infos = [];
		foreach ($items as &$i)
		{
			$formated_infos[] = self::getFormatedInfos($i, 'thumb');
		}
		Template::set('items', $formated_infos);

		return 1;
	}

	/**
	 * Retourne les informations EXIF d'un fichier.
	 *
	 * @param array $item_infos
	 *   Informations du fichier.
	 *
	 * @return array
	 */
	public static function getExif(array &$item_infos): array
	{
		$exif = [];
		if (Config::$params['exif'])
		{
			$exif_order = Config::$params['exif_order'];
			$exif_params = Config::$params['exif_params'];
			$filename = CONF_ALBUMS_PATH . '/' . $item_infos['item_path'];
			if (Utility::isJson($item_infos['item_exif']))
			{
				$data = ['exif' => Utility::jsonDecode($item_infos['item_exif'])];
				$metadata = new Metadata($filename, $data);
			}
			elseif (in_array($item_infos['item_type'], [Item::TYPE_JPEG, Item::TYPE_WEBP])
			&& file_exists($filename))
			{
				$metadata = new Metadata($filename);
			}
			if (isset($metadata))
			{
				$exif = $metadata->getExifFormated($exif_params);
				$exif = Metadata::getExifSorted($exif, $exif_order);
				foreach ($exif as $p => &$i)
				{
					if ($p == 'Make')
					{
						$i['value'] = Metadata::getExifCameraMake($i['value']);
						$i['link'] = App::getURL(
							'category/1/camera-brand/'
							. $item_infos['camera_brand_id'] . '-'
							. $item_infos['camera_brand_url']
						);
					}
					if ($p == 'Model')
					{
						$i['link'] = App::getURL(
							'category/1/camera-model/'
							. $item_infos['camera_model_id'] . '-'
							. $item_infos['camera_model_url']
						);
					}
				}
			}
		}
		return $exif;
	}

	/**
	 * Formate les informations d'un fichier.
	 *
	 * @param array $i
	 *   Informations brutes du fichier.
	 * @param string $origin
	 *   Origine : 'item' ou 'thumb'.
	 *
	 * @return array
	 */
	public static function getFormatedInfos(array &$i, string $origin = 'item'): array
	{
		$stats =
		[
			'comments' => [],
			'favorites' => [],
			'filesize' => [],
			'rating' => [],
			'views' => [],
			'votes' => []
		];

		$comments = function() use (&$i)
		{
			if (Config::$params['comments'])
			{
				return
				[
					'formated' => sprintf(
						$i['item_comments'] > 1 ? __('%s commentaires') : __('%s commentaire'),
						L10N::formatNumber((int) $i['item_comments'])
					),
					'short' => L10N::formatShortNumber((int) $i['item_comments']),
					'value' => $i['item_comments']
				];
			}
		};

		$date = function() use (&$i)
		{
			$formated = L10N::dt(__('%d/%m/%Y'), $i['item_adddt']);
			return
			[
				'formated' => $formated,
				'short' => $formated,
				'value' => $i['item_adddt']
			];
		};

		$favorites = function() use (&$i)
		{
			if (Config::$params['users'])
			{
				return
				[
					'formated' => sprintf(
						$i['item_favorites'] > 1 ? __('%s favoris') : __('%s favori'),
						L10N::formatNumber((int) $i['item_favorites'])
					),
					'short' => L10N::formatShortNumber((int) $i['item_favorites']),
					'value' => $i['item_favorites']
				];
			}
		};

		$filesize = function() use (&$i)
		{
			return
			[
				'formated' => $i['item_filesize']
					? L10N::formatFilesize($i['item_filesize'])
					: 0,
				'value' => $i['item_filesize']
			];
		};

		$rating = function() use (&$i)
		{
			if (Config::$params['votes'])
			{
				return
				[
					'array' => Rating::formatArray($i['item_rating']),
					'formated' => L10N::formatRating($i['item_rating']),
					'value' => $i['item_rating']
				];
			}
		};

		$views = function() use (&$i)
		{
			return
			[
				'formated' => sprintf(
					$i['item_hits'] > 1 ? __('%s vues') : __('%s vue'),
					L10N::formatNumber((int) $i['item_hits'])
				),
				'short' => L10N::formatShortNumber((int) $i['item_hits']),
				'value' => $i['item_hits']
			];
		};

		$votes = function() use (&$i)
		{
			if (Config::$params['votes'])
			{
				return
				[
					'formated' => sprintf(
						$i['item_votes'] > 1 ? __('%s votes') : __('%s vote'),
						L10N::formatNumber((int) $i['item_votes'])
					),
					'short' => L10N::formatShortNumber((int) $i['item_votes']),
					'value' => $i['item_votes']
				];
			}
		};

		foreach ($stats as $name => $data)
		{
			if ($origin == 'item' || Config::$params['thumbs_item_info_' . $name])
			{
				if ($r = ${$name}())
				{
					$stats[$name] = $r;
				}
			}
		}

		$date_added = $origin == 'item' || Config::$params['thumbs_item_info_date'];
		$size = $origin == 'item' || Config::$params['thumbs_item_info_size'];

		// Informations de vignettes pour les filtres.
		$stats['filter'] = [];
		if ($origin == 'thumb' && isset($_GET['filter'])
		&& (in_array($_GET['filter'], ['comments', 'favorites', 'votes', 'views'])
		|| substr($_GET['filter'], 0, 6) == 'recent'))
		{
			if (substr($_GET['filter'], 0, 6) == 'recent')
			{
				$stats['filter'] = $date();
				$date_added = FALSE;
			}
			else
			{
				$stats['filter'] = ${$_GET['filter']}();
			}
			if ($_GET['filter'] == 'votes')
			{
				$stats['rating'] = $rating();
			}
		}

		return
		[
			'filename' => basename($i['item_path']),
			'id' => $i['item_id'],
			'url' => $i['item_url'],

			// Sources et liens.
			'link' => self::getURLFilter('item/' . $i['item_id'] . '-' . $i['item_url'], TRUE),
			'link_admin' => App::getURLAdmin('item-edit/' . $i['item_id']),
			'source' => Item::isImage($i['item_type']) && (Config::$params['items_resize']
				|| (Config::$params['users'] && !Auth::$groupPerms['image_original']))
				? App::getImageResize($i)
				: App::getFileSource($i['item_path']),
			'source_download' => !empty(Template::$data['download_item'])
				? App::getFileSource($i['item_path'], TRUE)
				: '',

			// Type.
			'is_image' => Item::isImage($i['item_type']),
			'is_video' => Item::isVideo($i['item_type']),
			'type' => $i['item_type'],
			'type_mime' => Item::getTypeMime($i['item_type']),
			'type_text' => Item::getTypeText($i['item_type']),

			// Dates.
			'date_added' => $date_added
				? substr((string) $i['item_adddt'], 0, 10)
				: '',
			'date_added_short' => $date_added
				? L10N::dt(__('%d/%m/%Y'), $i['item_adddt'])
				: '',
			'date_added_text' => $origin == 'item'
				? mb_strtolower(L10N::dt(__('%A %d %B %Y'), $i['item_adddt']))
				: '',
			'datetime_added' => $date_added
				? $i['item_adddt']
				: '',
			'datetime_added_text' => $origin == 'item'
				? mb_strtolower(L10N::dt(__('%A %d %B %Y à %H:%M:%S'), $i['item_adddt']))
				: '',
			'date_created' => substr((string) $i['item_crtdt'], 0, 10),
			'date_created_link' => App::getURL('category/1/date-created/'
				. substr((string) $i['item_crtdt'], 0, 10)),
			'date_created_text' => $origin == 'item'
				? ($i['item_crtdt']
					? mb_strtolower(L10N::dt(__('%A %d %B %Y'), $i['item_crtdt']))
					: NULL)
				: '',
			'datetime_created' => $i['item_crtdt'],
			'datetime_created_text' => $origin == 'item'
				? ($i['item_crtdt']
					? mb_strtolower(L10N::dt(__('%A %d %B %Y à %H:%M:%S'), $i['item_crtdt']))
					: NULL)
				: '',
			'is_recent' => Config::$params['items_recent']
				&& strtotime($i['item_adddt']) > Item::getRecentTimestamp(),

			// Paramètres de l'utilisateur connecté.
			'user' =>
			[
				// Permission d'éditer le fichier ?
				'edit' => Auth::$isAdmin || (Config::$params['users']
					&& Auth::$id != 2 && $i['user_id'] == Auth::$id),

				// Dans les favoris ?
				'in_favorites' => isset($i['in_favorites'])
					? (bool) $i['in_favorites']
					: NULL,

				// Note.
				'rating' => $i['rating'] ?? NULL,
				'rating_array' => Rating::formatArray($i['rating'] ?? 0),
				'rating_formated' => L10N::formatRating($i['rating'] ?? 0)
			],

			// Utilisateur qui a ajouté le fichier.
			'owner' =>
			[
				'avatar_source' => Avatar::getURL(
					(int) $i['user_id'], (bool) $i['user_avatar']
				),
				'id' => $i['user_id'],
				'link' => empty(Auth::$groupPerms['members_profile'])
					? NULL
					: App::getURL('user/' . $i['user_id']),
				'nickname' => User::getNickname($i['user_login'], $i['user_nickname'])
			],

			// Métadonnées.
			'exif' => $i['exif'] ?? [],

			// Tags.
			'tags' => $i['tags'] ?? [],
			'tags_list' => $i['tags_list'] ?? [],

			// Géolocalisation.
			'geolocation_lat' => $i['item_lat'],
			'geolocation_long' => $i['item_long'],

			// Dimensions.
			'width' => $size ? $i['item_width'] : '',
			'height' => $size ? $i['item_height'] : '',

			// Durée.
			'duration' => $i['item_duration'],
			'duration_text' => $i['item_duration']
				? App::formatDuration($i['item_duration'])
				: '00:00',

			// Informations.
			'description' => $i['item_desc'],
			'title' => ($origin == 'item' || Config::$params['thumbs_item_info_title'])
				? $i['item_name']
				: '',

			// Statistiques.
			'stats' => $stats,

			// Vignette.
			'thumb_size' => function(int $max_width, int $max_height) use ($i):array
			{
				if (!$i['item_width'] || !$i['item_height'])
				{
					$i['item_width'] = 400;
					$i['item_height'] = 227;
				}
				return Image::getResizedSize((int) $i['item_width'],
					(int) $i['item_height'], $max_width, $max_height);
			},
			'thumb_src' => function($size = '', $quality = '', $poster = FALSE) use ($i)
			{
				return htmlspecialchars(
					App::getThumbSource(
						'item', $i, (string) $size, (string) $quality, (bool) $poster
					)
				);
			}
		];
	}

	/**
	 * Récupération des informations du fichier courant.
	 *
	 * @return int
	 */
	public static function getInfos(): int
	{
		$sql = "SELECT i.*,
					   cat.*,
					   %s,
					   'album' AS cat_type,
					   u.user_id,
					   u.user_avatar,
					   u.user_login,
					   u.user_nickname,
					   u.user_status,
					   cam_m.*,
					   cam_b.*,
					   (SELECT vote_rating
						  FROM {votes}
						 WHERE item_id = i.item_id
						   AND user_id = :user_id
						 LIMIT 1) AS rating,
					   (SELECT 1
						  FROM {favorites}
						 WHERE item_id = i.item_id
						   AND user_id = :user_id
						 LIMIT 1) AS in_favorites
				  FROM {items} AS i
		     LEFT JOIN {categories} AS cat
				    ON i.cat_id = cat.cat_id
		     LEFT JOIN {users} AS u
				    ON i.user_id = u.user_id
		     LEFT JOIN {cameras_items} AS cam_i
				    ON cam_i.item_id = i.item_id
		     LEFT JOIN {cameras_models} AS cam_m
				    ON cam_m.camera_model_id = cam_i.camera_model_id
		     LEFT JOIN {cameras_brands} AS cam_b
				    ON cam_b.camera_brand_id = cam_m.camera_brand_id
			     WHERE %s
				   AND item_status = '1'
				   AND i.item_id = :item_id";
		$sql = sprintf($sql, SQL::catPassword('select'), SQL::catPerms());
		$params = ['item_id' => $_GET['item_id'], 'user_id' => Auth::$id];
		if (!DB::execute($sql, $params))
		{
			return -1;
		}
		if (!isset((self::$infos = DB::fetchRow())['item_id']))
		{
			return 0;
		}

		// L'utilisateur a-t-il entré le bon mot de passe ?
		if (!self::checkCookiePassword(self::$infos))
		{
			return 0;
		}

		// Fil d'Ariane.
		Template::breadcrumb(self::$infos, TRUE);
		unset(Template::$data['breadcrumb'][0]);

		// Réglages.
		self::$infos['cat_parents'] .= self::$infos['cat_id'] . Category::PARENTS_SEPARATOR;
		self::$infos = [self::$infos];
		Parents::settings(self::$infos);
		self::$infos = self::$infos[0];

		// Informations EXIF.
		self::$infos['exif'] = self::getExif(self::$infos);

		// Tags.
		if (Config::$params['tags'])
		{
			$tags = self::getTags(self::$infos);
			self::$infos['tags'] = $tags;
			$tags_list = [];
			foreach ($tags as &$i)
			{
				$tags_list[] = $i['name'];
			}
			self::$infos['tags_list'] = $tags_list;
		}

		// Géolocalisation.
		if (Config::$params['geolocation'])
		{
			if (self::$infos['item_lat'] === NULL || self::$infos['item_long'] === NULL)
			{
				Template::set('geolocation', 0);
			}
		}

		// Émojis.
		Template::set('comments_emoji_picker', 0);
		if (Config::$params['comments_emoji_picker'])
		{
			Template::set('comments_emoji_picker', 1);
			$format = function(array $list): array
			{
				$r = [];
				foreach ($list as &$chars)
				{
					$r = array_merge($r, $chars);
				}
				$r = array_flip($r);
				foreach ($r as $code => &$text)
				{
					$t = __((string) $code);
					$text = $t == $code ? '' : $t;
				}
				return $r;
			};

			Template::set('emoji_smileys_emotion', $format(Emoji::LIST['smileys-and-emotion']));
			Template::set('emoji_people_body', $format(Emoji::LIST['people-and-body']));
			Template::set('emoji_animals_nature', $format(Emoji::LIST['animals-and-nature']));
			Template::set('emoji_food_drink', $format(Emoji::LIST['food-and-drink']));
			Template::set('emoji_travel_places', $format(Emoji::LIST['travel-and-places']));
			Template::set('emoji_activities', $format(Emoji::LIST['activities']));
			Template::set('emoji_objects', $format(Emoji::LIST['objects']));
			Template::set('emoji_symbols', $format(Emoji::LIST['symbols']));
		}

		// Nombre de vues.
		if (Item::hit((int) $_GET['item_id']) > 0)
		{
			self::$infos['item_hits']++;
		}

		// Téléchargement.
		Template::set('download_item', (int) self::itemDownload(self::$infos));

		// Commentaires.
		self::$infos['commentable'] = self::$infos['parent_commentable']
			&& self::$infos['item_commentable'];
		Template::set('comments', Config::$params['comments']);
		Template::set('commentable', self::$infos['commentable']);

		// Votes.
		self::$infos['votable'] = self::$infos['parent_votable']
			&& self::$infos['item_votable'];
		Template::set('votes', Config::$params['votes']);
		Template::set('votes_add', !Config::$params['users']
			|| (Config::$params['users'] && Auth::$groupPerms['votes']));
		Template::set('votable', self::$infos['votable']);

		// Permission d'édition pour les administrateurs
		// et le propriétaire du fichier.
		Template::set('item_edit', Auth::$isAdmin
			|| (Config::$params['users'] && self::$infos['user_id'] == Auth::$id));

		// Formatage des informations.
		Template::set('item', self::getFormatedInfos(self::$infos, 'item'));
		Template::set('category', GalleryCategory::getFormatedInfos(self::$infos));

		// URL canonique.
		Template::set('canonical', GALLERY_HOST . App::getURL('item/'
			. self::$infos['item_id'] . '-' . self::$infos['item_url']));

		// Description pour balise <meta>.
		Template::set('meta_description',
			App::getMetaDescription((string) self::$infos['item_desc']));

		// Pagination.
		if (is_int($r = self::_itemPagination(self::$infos)))
		{
			return $r;
		}

		// Flux RSS.
		if (Config::$params['rss']
		&& Config::$params['comments'] && self::$infos['commentable'])
		{
			self::_rss('item', self::$infos);
		}

		return 1;
	}

	/**
	 * Retourne les tags liés à un fichier.
	 *
	 * @param array $item_infos
	 *   Informations du fichier.
	 *
	 * @return array
	 */
	public static function getTags(array &$item_infos): array
	{
		$tags = [];
		if (Config::$params['tags'])
		{
			$sql = "SELECT t.*
					  FROM {tags} AS t,
						   {tags_items} AS ti
					 WHERE t.tag_id = ti.tag_id
					   AND ti.item_id = ?";
			if (DB::execute($sql, (int) $item_infos['item_id']))
			{
				foreach (DB::fetchAll() as &$tag)
				{
					$tags[$tag['tag_name']] =
					[
						'link' => App::getURL(
							'category/1-' . __('galerie') .
							'/tag/' . $tag['tag_id'] . '-' . $tag['tag_url']
						),
						'name' => $tag['tag_name']
					];
				}
				uksort($tags, 'Utility::alphaSort');
			}
		}
		return array_values($tags);
	}

	/**
	 * Détermine si un fichier peut être téléchargé
	 * en fonction des permissions sur ce fichier et des
	 * paramètres de configuration.
	 *
	 * @param array $i
	 *   Informations du fichier.
	 *
	 * @return bool
	 */
	public static function itemDownload(array &$i): bool
	{
		return $i['parent_downloadable']
			&& $i['item_downloadable']
			&& Config::$params['download_item'] &&
			 (!Config::$params['users'] ||
			  (Config::$params['users'] && (Item::isImage($i['item_type'])
				? (bool) Auth::$groupPerms['image_original'] : TRUE)));
	}



	/**
	 * Récupération de fichiers.
	 *
	 * @param int $items_per_page
	 *   Nombre de fichiers par page (section "album").
	 *
	 * @return array|int
	 *   Retourne un tableau des informations des fichiers,
	 *   -1 en cas d'erreur ou 0 si aucun fichier n'a été trouvé.
	 */
	private static function _getItems(int $items_per_page = 0)
	{
		Template::set('objects_count', 0);

		// Clause WHERE.
		$sql_where = "item_status = '1' AND %s AND %s";
		$params = [];

		// Filtres.
		if (isset($_GET['filter']))
		{
			if (($_GET['filter'] == 'search' && $where = self::$search->sql())
			|| $where = SQL::itemsWhere($_GET['filter'], $_GET['filter_value']))
			{
				$sql_where .= " AND " . $where['sql'];
				$params = $where['params'];
			}
			else if ($_GET['filter'] == 'search')
			{
				return 1;
			}
		}

		// Limitation à la catégorie courante.
		if (isset($_GET['filter']) && $_GET['section'] == 'item')
		{
			if ($_GET['filter_cat_id'] > 1)
			{
				$sql = "SELECT cat_id AS id,
							   cat_url AS urlname,
							   cat_name AS title,
							   cat_path,
							   CASE WHEN cat_filemtime IS NULL
									THEN 'category' ELSE 'album'
									 END AS type
						  FROM {categories}
						 WHERE cat_id = ?";
				if (DB::execute($sql, $_GET['filter_cat_id']))
				{
					if (!$category_filter = DB::fetchRow())
					{
						return 0;
					}
					$sql_where .= " AND item_path LIKE :path";
					$params['path'] = DB::likeEscape($category_filter['cat_path']) . '/%';
					unset($category_filter['cat_path']);
					Template::set('category_filter', $category_filter);
				}
			}
		}
		elseif (Template::$data['category']['id'] > 1)
		{
			if (isset($_GET['filter']))
			{
				$sql_where .= " AND item_path LIKE :path";
				$params['path'] = DB::likeEscape(Template::$data['category']['path']) . '/%';
			}
			else
			{
				$sql_where .= " AND i.cat_id = :cat_id";
				$params['cat_id'] = (int) Template::$data['category']['id'];
			}
		}

		// Clause ORDER BY.
		$sql_order_by = SQL::itemsOrderBy(
			$_GET['filter'] ?? '',
			self::$infos['cat_orderby'] ??
			GalleryCategory::$infos['cat_orderby'] ??
			Config::$params['items_sql_order_by'],
			GalleryCategory::$infos
		);

		// Clause FROM.
		$sql_from = SQL::itemsFrom($_GET['filter'] ?? '');

		// Récupération des fichiers.
		if ($items_per_page)
		{
			$start = ($items_per_page * ($_GET['page'] - 1));
			$sql = "SELECT i.*,
						   cat.cat_parents,
						   u.user_avatar,
						   u.user_login,
						   u.user_nickname,
						   u.user_status
					  FROM {items} AS i
						   $sql_from
				 LEFT JOIN {categories} AS cat
						ON i.cat_id = cat.cat_id
				 LEFT JOIN {users} AS u
						ON i.user_id = u.user_id
					 WHERE $sql_where
				  ORDER BY $sql_order_by
					 LIMIT $start,$items_per_page";
		}
		else
		{
			$sql = "SELECT i.item_id,
						   i.item_url,
						   i.item_type
					  FROM {items} AS i
						   $sql_from
				 LEFT JOIN {categories} AS cat
						ON i.cat_id = cat.cat_id
					 WHERE $sql_where
				  ORDER BY $sql_order_by";
		}
		$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
		if (!DB::execute($sql, $params))
		{
			return -1;
		}
		if (!$items = DB::fetchAll())
		{
			if ($_GET['page'] > 1 && $_GET['section'] != 'item')
			{
				App::redirect($_GET['q_pageless']);
			}
			elseif (isset($_GET['filter']) && $_GET['section'] == 'item')
			{
				App::redirect('category/1/' . $_GET['filter']
					. ($_GET['filter_value'] == $_GET['filter_cat_id']
					? '' : '/' . $_GET['filter_value']));
			}
			return (int) (isset($_GET['filter'])
				&& !strstr($_GET['filter'], 'camera')
				&& !strstr($_GET['filter'], 'date')
				&& !strstr($_GET['filter'], 'tags')
				&& !strstr($_GET['filter'], 'user'));
		}

		// Si le fichier courant ne se trouve pas dans la liste
		// des fichiers correspondant au filtre actuel, on redirige
		// vers la page du fichier sans le filtre.
		if ($_GET['section'] == 'item' && isset($_GET['filter'])
		&& array_search(Template::$data['item']['id'], array_column($items, 'item_id')) === FALSE)
		{
			App::redirect($_GET['q_filterless']);
			return 0;
		}

		// Nombre de fichiers.
		if ($items_per_page)
		{
			if (isset($_GET['filter']))
			{
				unset($params['user_id']);
				$sql = "SELECT COUNT(*)
						  FROM {items} AS i
							   $sql_from
					 LEFT JOIN {categories} AS cat
							ON i.cat_id = cat.cat_id
						 WHERE $sql_where";
				$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
				if (DB::execute($sql, $params))
				{
					Template::set('objects_count', DB::fetchVal());
				}
			}
			else
			{
				Template::set('objects_count', Template::$data['category']['objects_count']);
			}
		}

		return $items;
	}

	/**
	 * Informations pour la pagination entre les pages de fichiers.
	 *
	 * @param array $item_infos
	 *
	 * @return mixed
	 */
	private static function _itemPagination(array $item_infos)
	{
		// Récupération des informations des fichiers.
		if (is_int($all_items = self::_getItems()))
		{
			return $all_items;
		}

		// Nombre de fichiers.
		$items_count = count($all_items);
		Template::set('objects_count', $items_count);

		// Informations de template.
		foreach ($all_items as &$cols)
		{
			unset($cols['item_type']);
			unset($cols['password_auth']);
		}
		$link = function($i) use (&$all_items)
		{
			return isset($all_items[$i])
				? self::getURLFilter('item/' . $all_items[$i]['item_id']
					. '-' . $all_items[$i]['item_url'], TRUE)
				: '';
		};
		for ($i = 0; $i < $items_count; $i++)
		{
			if ($item_infos['item_id'] == $all_items[$i]['item_id'])
			{
				Template::set('pages',
				[
					'count' => (int) $items_count,
					'first' =>
					[
						'active' => $i > 0,
						'link' => $link(0)
					],
					'prev' =>
					[
						'active' => $i > 0,
						'link' => $link($i - 1)
					],
					'next' =>
					[
						'active' => $i + 1 < $items_count,
						'link' => $link($i + 1)
					],
					'last' =>
					[
						'active' => $i + 1 < $items_count,
						'link' => $link($items_count - 1)
					]
				]);
				break;
			}
		}

		// Position du fichier.
		$item_position = array_search(
			[
				'item_id' => self::$infos['item_id'],
				'item_url' => self::$infos['item_url']
			],
			$all_items
		) + 1;
		Template::set('item_position', $item_position);

		// Page parente pour l'album du fil d'Ariane.
		$index = 0;
		foreach (Template::$data['breadcrumb'] as $k => &$i)
		{
			if ($i['id'] == Template::$data['category']['id'])
			{
				$index = $k;
				break;
			}
		}
		if ($index && !isset($_GET['filter']))
		{
			$items_per_page = (int) Config::$params['thumbs_item_nb_per_page'];
			$parent_page = ceil($item_position / $items_per_page);
			if ($parent_page > 1)
			{
				Template::$data['breadcrumb'][$index]['url'] .= '/page/' . $parent_page;
			}
		}
	}
}
?>